﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public abstract class Attachable : MonoBehaviour
{
    public bool Final;
    public HingeJoint2D Hinge;
    abstract public bool IsArm { get; }

    private void Awake()
    {
        Hinge = gameObject.GetComponent<HingeJoint2D>();
        Hinge.enabled = false;
    }

    private void OnCollisionEnter2D(Collision2D collision)
    {
        GameObject player = Hinge?.connectedBody?.gameObject;

        if (player != null && LayerMask.LayerToName(collision.gameObject.layer) == "Damaging")
        {
            // Ignore if bullet was fired by this same player
            if (player.name == collision.gameObject.GetComponent<ProjectileController>().Player.name)
                return;

            JointController[] joints = player.GetComponentsInChildren<JointController>();

            foreach(var joint in joints)
            {
                if(joint.RobotPart == transform)
                {
                    joint.DetachPart();
                }
            }
        }

        if(player == null)
        {
            if(LayerMask.LayerToName(collision.gameObject.layer) == "Player")
            {
                player = collision.gameObject;

                JointController[] joints = player.GetComponentsInChildren<JointController>();

                foreach (var joint in joints)
                {
                    bool attachedSuccessfully = joint.TryAttachPart(transform);

                    if (attachedSuccessfully)
                    {
                        break;
                    }
                }
            }
            else if (LayerMask.LayerToName(collision.gameObject.layer) == "Part")
            {
                var part = collision.gameObject.GetComponent<Attachable>();

                player = part.Hinge?.connectedBody?.gameObject;

                if (player != null)
                {
                    JointController[] joints = player.GetComponentsInChildren<JointController>();

                    foreach (var joint in joints)
                    {
                        bool attachedSuccessfully = joint.TryAttachPart(transform);

                        if (attachedSuccessfully)
                        {
                            break;
                        }
                    }
                }
            }
        }
    }
}