﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class JointController : MonoBehaviour
{
    //TEMP
    public Transform TempRobotPart;

    public bool IsArm;
    public float minInput = 0.1f;

    public Transform RobotPart;
    Vector2 AimDirection;

    private void Update()
    {
        if(RobotPart != null)
            Aim();
    }

    public bool HasFinalAttached()
    {
        return
            transform.childCount > 0 &&
            transform.GetChild(0).GetComponent<Attachable>().Final;
    }

    [ContextMenu("Attach")]
    public void AttachTest()
    {
        TryAttachPart(TempRobotPart);
    }

    public bool TryAttachPart(Transform robotPart)
    {
        if (RobotPart == null)
        {
            var attachable = robotPart.GetComponent<Attachable>();

            if (attachable != null && attachable.IsArm == IsArm)
            {
                RobotPart = robotPart;
                RobotPart.transform.position = transform.position;
                RobotPart.transform.rotation = Quaternion.Euler(0, 0, 0);

                // Attaching to the player body
                RobotPart.GetComponent<Attachable>().Hinge.enabled = true;
                RobotPart.GetComponent<Attachable>().Hinge.connectedBody = transform.parent.GetComponent<Rigidbody2D>();
                return true;
            }
        }

        return false;
    }

    [ContextMenu("Detach")]
    public void DetachPart()
    {
        if (RobotPart != null)
        {
            RobotPart.GetComponent<Attachable>().Hinge.connectedBody = null;
            RobotPart.GetComponent<Attachable>().Hinge.enabled = false;
            RobotPart = null;
        }
    }


    /////// TODO: make subclasses of Joint controller for arm and bottom and split the following methods accordingly
    public float JumpForce()
    {
        if (RobotPart != null)
        {
            Bottom bottom = RobotPart.GetComponent<Bottom>();

            if (bottom != null && bottom.Grounded)
            {
                return bottom.JumpForce;
            }
        }
        return 0;
    }

    public float SpeedModifier()
    {
        if (RobotPart != null
            && RobotPart.GetComponent<Bottom>() != null)
        {
            return RobotPart.GetComponent<Bottom>().SpeedModifier;
        }
        return 1;
    }

    public void Shoot(GameObject shooter)
    {
        if (RobotPart != null
            && RobotPart.GetComponent<Arm>() != null)
        {
            RobotPart.GetComponent<Arm>().Shoot(AimDirection, shooter);
        }
    }

    private void Aim()
    {
        if (RobotPart != null &&
            RobotPart.GetComponent<Arm>() != null)
        {
            //float horizontal = Input.GetKey(KeyCode.F) ? -1 : Input.GetKey(KeyCode.H) ? 1 : 0;
            //float vertical = Input.GetKey(KeyCode.G) ? -1 : Input.GetKey(KeyCode.T) ? 1 : 0;

            //Vector2 direction = new Vector2(horizontal, vertical);

            Vector2 direction = transform.parent.GetComponent<Player>().AimInputValue;
            if (direction.magnitude > minInput)
            {
                RobotPart.transform.rotation = Quaternion.Euler(0, 0, Vector2.SignedAngle(new Vector2(1, 0), direction));
                AimDirection = direction;
            }
            else
            {
                RobotPart.transform.rotation = Quaternion.Euler(0, 0, Vector2.SignedAngle(new Vector2(1, 0), AimDirection));
            }
        }
    }
}
