﻿using UnityEngine;
using UnityEngine.Events;

using UnityEngine.InputSystem;
using UnityEngine.SceneManagement;

public class Player : MonoBehaviour
{
    [SerializeField] float BaseSpeed = 1;
    [SerializeField] float SmoothTime = 0.05f;
    float JumpForce;
    [SerializeField] float FastFallSpeed = 1;
    [SerializeField] float FlySpeed = 0.8f;
    [SerializeField] float TimeToLoadScene = 2;

    JointController LeftJoint;
    JointController RightJoint;
    JointController BottomJoint;

    public Vector2 AimInputValue;

    public float recoil = -1000.0f;
    public Vector2 Movement;

    Rigidbody2D Rb2D;
    Vector3 Velocity = Vector3.zero;

    private void Awake()
    {
        Rb2D = GetComponent<Rigidbody2D>();

        foreach(Transform child in transform)
        {
            if (child.name.Contains("Left"))
            {
                LeftJoint = child.GetComponent<JointController>();
            }
            else if (child.name.Contains("Right"))
            {
                RightJoint = child.GetComponent<JointController>();
            }
            else if (child.name.Contains("Bottom"))
            {
                BottomJoint = child.GetComponent<JointController>();
            }
            else
            {
                Debug.Log($"Failed to attach {child.name}");
            }
        }
    }

    private void OnMove(InputValue value)
    {
        Movement = value.Get<Vector2>();
    }

    private void OnJump()
    {
        Jump();
    }

    private void OnShootLeft()
    {
        // TODO: Add Recoil
        // Rb2D.AddForce(recoil * AimDirectionLeft);

        // LeftWeaponGO.GetComponent<Weapon>().Shoot(AimDirectionLeft);
        LeftJoint?.Shoot(gameObject);
    }

    private void OnShootRight()
    {
        RightJoint?.Shoot(gameObject);
        // RightWeaponGO.GetComponent<Weapon>().Shoot(AimDirectionRight);

        // Recoil
        // Rb2D.AddForce(recoil * AimDirectionRight);
    }

    private void OnAim(InputValue value)
    {
        Vector2 dir = value.Get<Vector2>();
        AimInputValue = dir;

        // Debug.Log(AimInputValue.magnitude);
    }

    void FixedUpdate()
    {
        Move(Movement);
    }

    private void OnCollisionEnter2D(Collision2D collision)
    {
        if (LayerMask.LayerToName(collision.gameObject.layer) == "Goal")
        {
            Debug.Log($"{transform.name} won!");
            //Play success sound
            Invoke("ReloadScene", TimeToLoadScene);
        }
    }

    private void ReloadScene()
    {
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    private void Move(Vector2 dir)
    {
        // float horizontalMovement = Input.GetAxis("Horizontal");
        // float v = Input.GetAxis("Vertical");
        float h = dir.x;
        float v = dir.y;

        float bottomSpeedModifier = BottomJoint.SpeedModifier();
        float targetHorizontalVelocity = h * BaseSpeed * bottomSpeedModifier;
        float targetVerticalVelocity = Rb2D.velocity.y;

        // Player can always fast-fall, if he has all 3 final parts, he can also levitate/fly.
        if (v < 0)
        {
            targetVerticalVelocity += v * FastFallSpeed;
        }
        else if (LeftJoint.HasFinalAttached() && RightJoint.HasFinalAttached() && BottomJoint.HasFinalAttached())
        {
            targetVerticalVelocity += v * FlySpeed;
        }

        Vector3 targetVector = new Vector2(targetHorizontalVelocity, targetVerticalVelocity);
        Rb2D.velocity = Vector3.SmoothDamp(Rb2D.velocity, targetVector, ref Velocity, SmoothTime);
    }

    private void Jump()
    {
        Rb2D.AddForce(new Vector2(0f, BottomJoint.JumpForce()));
    }
}