﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ProjectileController : MonoBehaviour
{
    public float BoostForce;
    public GameObject RobotExplosion;
    public GameObject RocketExplosion;

    public AudioClip[] FireAudioClips;
    public AudioClip[] DamageAudioClips;

    public AudioClip RocketBoomClip;

    private AudioSource audioSource;
    public GameObject Player;
    public GameObject Weapon;

    // public float minSpeed;

    private Rigidbody2D rb2d;

    private void Awake()
    {
        rb2d = GetComponent<Rigidbody2D>();
        audioSource = GetComponent<AudioSource>();
    }

    private void Start()
    {
        // Play bullet fired sounds
        if (audioSource != null && FireAudioClips != null && FireAudioClips.Length > 0)
        {
            SoundManager.instance.RandomizeSfx(audioSource, FireAudioClips);
        }
    }

    public void Boost(Vector2 direction, GameObject player, GameObject weapon)
    {
        Player = player;
        Weapon = weapon;

        transform.rotation = Quaternion.Euler(0, 0, Vector2.SignedAngle(new Vector2(1, 0), direction)); 
        rb2d.AddForce(direction * BoostForce);
    }


    private void OnCollisionEnter2D(Collision2D collision)
    {
        // Ignore collisions with object that shot the projectile
        if (collision.gameObject == Weapon || collision.gameObject == Player)
            return;

        if(gameObject.name.Contains("Rocket") && RocketExplosion != null)
        {
            Instantiate(RocketExplosion, transform.position, transform.rotation);
            SoundManager.instance.RandomizeSfx(audioSource, RocketBoomClip);

        }

        if (LayerMask.LayerToName(collision.gameObject.layer) == "Player")
        {
            // TODO: Damage code here
            Instantiate(RobotExplosion, transform.position, transform.rotation);

            if (audioSource != null && DamageAudioClips != null && DamageAudioClips.Length > 0)
            {
                SoundManager.instance.RandomizeSfx(audioSource, DamageAudioClips);
            }
        }
        else
        {
            // Could also bounce, but add that later
            //Destroy(gameObject);

        }

        // Make bullet disappear
        GetComponent<SpriteRenderer>().enabled = false;
        //GetComponent<Collider>().enabled = false;

        Destroy(gameObject, 2.0f);
    }
}
